@@ -9,14 +9,19 @@ from .utils import func_args_preprocessing
 
 class CoinGeckoAPI:
     __API_URL_BASE = 'https://api.coingecko.com/api/v3/'
+    __PRO_API_URL_BASE = 'https://pro-api.coingecko.com/api/v3/'
 
-    def __init__(self, api_base_url=__API_URL_BASE):
-        self.api_base_url = api_base_url
+    def __init__(self, api_key: str = '', retries=5):
+        self.api_key = api_key
+        if api_key:
+            self.api_base_url = self.__PRO_API_URL_BASE
+        else:
+            self.api_base_url = self.__API_URL_BASE
         self.request_timeout = 120
 
         self.session = requests.Session()
-        retries = Retry(total=5, backoff_factor=0.5, status_forcelist=[502, 503, 504])
-        self.session.mount('http://', HTTPAdapter(max_retries=retries))
+        retries = Retry(total=retries, backoff_factor=0.5, status_forcelist=[502, 503, 504])
+        self.session.mount('https://', HTTPAdapter(max_retries=retries))
 
     def __request(self, url):
         # print(url)
@@ -41,6 +46,10 @@ class CoinGeckoAPI:
             raise
 
     def __api_url_params(self, api_url, params, api_url_has_params=False):
+        # if using pro version of CoinGecko, inject key in every call
+        if self.api_key:
+            params['x_cg_pro_api_key'] = self.api_key
+
         if params:
             # if api_url contains already params and there is already a '?' avoid
             # adding second '?' (api_url += '&' if '?' in api_url else '?'); causes
@@ -56,10 +65,12 @@ class CoinGeckoAPI:
         return api_url
 
     # ---------- PING ----------#
-    def ping(self):
+    def ping(self, **kwargs):
         """Check API server status"""
 
         api_url = '{0}ping'.format(self.api_base_url)
+        api_url = self.__api_url_params(api_url, kwargs)
+
         return self.__request(api_url)
 
     # ---------- SIMPLE ----------#
@@ -179,14 +190,14 @@ class CoinGeckoAPI:
 
         return self.__request(api_url)
 
-    @func_args_preprocessing
-    def get_coin_status_updates_by_id(self, id, **kwargs):
-        """Get status updates for a given coin"""
-
-        api_url = '{0}coins/{1}/status_updates'.format(self.api_base_url, id)
-        api_url = self.__api_url_params(api_url, kwargs)
-
-        return self.__request(api_url)
+    # @func_args_preprocessing
+    # def get_coin_status_updates_by_id(self, id, **kwargs):
+    #     """Get status updates for a given coin"""
+    #
+    #     api_url = '{0}coins/{1}/status_updates'.format(self.api_base_url, id)
+    #     api_url = self.__api_url_params(api_url, kwargs)
+    #
+    #     return self.__request(api_url)
 
     @func_args_preprocessing
     def get_coin_ohlc_by_id(self, id, vs_currency, days, **kwargs):
@@ -225,7 +236,7 @@ class CoinGeckoAPI:
 
         api_url = '{0}coins/{1}/contract/{2}/market_chart/range?vs_currency={3}&from={4}&to={5}'.format(
             self.api_base_url, id, contract_address, vs_currency, from_timestamp, to_timestamp)
-        api_url = self.__api_url_params(api_url, kwargs)
+        api_url = self.__api_url_params(api_url, kwargs, api_url_has_params=True)
 
         return self.__request(api_url)
 
@@ -295,14 +306,14 @@ class CoinGeckoAPI:
 
         return self.__request(api_url)
 
-    @func_args_preprocessing
-    def get_exchanges_status_updates_by_id(self, id, **kwargs):
-        """Get status updates for a given exchange"""
-
-        api_url = '{0}exchanges/{1}/status_updates'.format(self.api_base_url, id)
-        api_url = self.__api_url_params(api_url, kwargs)
-
-        return self.__request(api_url)
+    # @func_args_preprocessing
+    # def get_exchanges_status_updates_by_id(self, id, **kwargs):
+    #     """Get status updates for a given exchange"""
+    #
+    #     api_url = '{0}exchanges/{1}/status_updates'.format(self.api_base_url, id)
+    #     api_url = self.__api_url_params(api_url, kwargs)
+    #
+    #     return self.__request(api_url)
 
     @func_args_preprocessing
     def get_exchanges_volume_chart_by_id(self, id, days, **kwargs):
@@ -315,24 +326,24 @@ class CoinGeckoAPI:
 
         return self.__request(api_url)
 
-    # ---------- FINANCE ----------#
-    @func_args_preprocessing
-    def get_finance_platforms(self, **kwargs):
-        """Get cryptocurrency finance platforms data"""
-
-        api_url = '{0}finance_platforms'.format(self.api_base_url)
-        api_url = self.__api_url_params(api_url, kwargs)
-
-        return self.__request(api_url)
-
-    @func_args_preprocessing
-    def get_finance_products(self, **kwargs):
-        """Get cryptocurrency finance products data"""
-
-        api_url = '{0}finance_products'.format(self.api_base_url)
-        api_url = self.__api_url_params(api_url, kwargs)
-
-        return self.__request(api_url)
+    # # ---------- FINANCE ----------#
+    # @func_args_preprocessing
+    # def get_finance_platforms(self, **kwargs):
+    #     """Get cryptocurrency finance platforms data"""
+    #
+    #     api_url = '{0}finance_platforms'.format(self.api_base_url)
+    #     api_url = self.__api_url_params(api_url, kwargs)
+    #
+    #     return self.__request(api_url)
+    #
+    # @func_args_preprocessing
+    # def get_finance_products(self, **kwargs):
+    #     """Get cryptocurrency finance products data"""
+    #
+    #     api_url = '{0}finance_products'.format(self.api_base_url)
+    #     api_url = self.__api_url_params(api_url, kwargs)
+    #
+    #     return self.__request(api_url)
 
     # ---------- INDEXES ----------#
     @func_args_preprocessing
@@ -408,43 +419,43 @@ class CoinGeckoAPI:
 
         return self.__request(api_url)
 
-    # ---------- STATUS UPDATES ----------#
-    @func_args_preprocessing
-    def get_status_updates(self, **kwargs):
-        """List all status_updates with data (description, category, created_at, user, user_title and pin)"""
+    # # ---------- STATUS UPDATES ----------#
+    # @func_args_preprocessing
+    # def get_status_updates(self, **kwargs):
+    #     """List all status_updates with data (description, category, created_at, user, user_title and pin)"""
+    #
+    #     api_url = '{0}status_updates'.format(self.api_base_url)
+    #     api_url = self.__api_url_params(api_url, kwargs)
+    #
+    #     return self.__request(api_url)
 
-        api_url = '{0}status_updates'.format(self.api_base_url)
-        api_url = self.__api_url_params(api_url, kwargs)
-
-        return self.__request(api_url)
-
-    # ---------- EVENTS ----------#
-    @func_args_preprocessing
-    def get_events(self, **kwargs):
-        """Get events, paginated by 100"""
-
-        api_url = '{0}events'.format(self.api_base_url)
-        api_url = self.__api_url_params(api_url, kwargs)
-
-        return self.__request(api_url)
-
-    @func_args_preprocessing
-    def get_events_countries(self, **kwargs):
-        """Get list of event countries"""
-
-        api_url = '{0}events/countries'.format(self.api_base_url)
-        api_url = self.__api_url_params(api_url, kwargs)
-
-        return self.__request(api_url)
-
-    @func_args_preprocessing
-    def get_events_types(self, **kwargs):
-        """Get list of event types"""
-
-        api_url = '{0}events/types'.format(self.api_base_url)
-        api_url = self.__api_url_params(api_url, kwargs)
-
-        return self.__request(api_url)
+    # # ---------- EVENTS ----------#
+    # @func_args_preprocessing
+    # def get_events(self, **kwargs):
+    #     """Get events, paginated by 100"""
+    #
+    #     api_url = '{0}events'.format(self.api_base_url)
+    #     api_url = self.__api_url_params(api_url, kwargs)
+    #
+    #     return self.__request(api_url)
+    #
+    # @func_args_preprocessing
+    # def get_events_countries(self, **kwargs):
+    #     """Get list of event countries"""
+    #
+    #     api_url = '{0}events/countries'.format(self.api_base_url)
+    #     api_url = self.__api_url_params(api_url, kwargs)
+    #
+    #     return self.__request(api_url)
+    #
+    # @func_args_preprocessing
+    # def get_events_types(self, **kwargs):
+    #     """Get list of event types"""
+    #
+    #     api_url = '{0}events/types'.format(self.api_base_url)
+    #     api_url = self.__api_url_params(api_url, kwargs)
+    #
+    #     return self.__request(api_url)
 
     # ---------- EXCHANGE-RATES ----------#
     @func_args_preprocessing
@@ -456,6 +467,16 @@ class CoinGeckoAPI:
 
         return self.__request(api_url)
 
+    # ---------- SEARCH ----------#
+    @func_args_preprocessing
+    def search(self, query, **kwargs):
+        """Search for coins, categories and markets on CoinGecko"""
+
+        api_url = '{0}search?query={1}'.format(self.api_base_url, query)
+        api_url = self.__api_url_params(api_url, kwargs, api_url_has_params=True)
+
+        return self.__request(api_url)
+
     # ---------- TRENDING ----------#
     @func_args_preprocessing
     def get_search_trending(self, **kwargs):
@@ -493,4 +514,4 @@ class CoinGeckoAPI:
         api_url = '{0}companies/public_treasury/{1}'.format(self.api_base_url, coin_id)
         api_url = self.__api_url_params(api_url, kwargs)
 
-        return self.__request(api_url)
+        return self.__request(api_url)
\ No newline at end of file
